#!/usr/bin/python3
# -*- coding: utf-8 -*-

# The code is derived from https://github.com/enakai00/colab_tfbook
# and modified by izumi@ieee.org, 2021

# このコードはこの↓書籍のサンプルコードをベースに改変したものである
# 『TensorFlow と Keras で 動かしながら学ぶ ディープラーニングの仕組み』
# 中井悦司, マイナビ出版, 2019年
# https://book.mynavi.jp/ec/products/detail/id=109925

# 別スレッドでの predict がうまく走らない（一発目は走っても二発目でコケる）
# →別プロセスとして起動する

import os
import flask
import subprocess

app = flask.Flask(__name__, static_folder='ex5-6')


@app.route('/', methods=['GET'])
def page_write():
    html = '''<!DOCTYPE html>
<html>
<body>

<table>
<td style="border-style: none;">
<div style="border: solid 2px #666; width: 143px; height: 144px;">
<canvas width="140" height="140"></canvas>
</div></td>
<td style="border-style: none;">
<button onclick="clear_value()">Clear</button>
<button onclick="set_value()">Done</button>
</td>
</table>

<script type="text/Javascript">
  var pixels = [];
  for (var i = 0; i < 28*28; i++) pixels[i] = 0;
  var click = 0;

  var canvas = document.querySelector("canvas");
  canvas.addEventListener("mousemove", function(e) {
      if (e.buttons == 1) {
          click = 1;
          canvas.getContext("2d").fillStyle = "rgb(0, 0, 0)";
          canvas.getContext("2d").fillRect(e.offsetX, e.offsetY, 8, 8);
          x = Math.floor(e.offsetY*0.2);
          y = Math.floor(e.offsetX*0.2) + 1;
          for (var dy = 0; dy < 2; dy++) {
              for (var dx = 0; dx < 2; dx++) {
                  if ((x + dx < 28) && (y + dy < 28)) {
                      pixels[(y+dy)+(x+dx)*28] = 1;
                  }
              }
          }
      } else {
          // if (click == 1) set_value()
          click = 0;
      }
  });

  function set_value() {
      var result = "";
      for (var i = 0; i < 28*28; i++) result += pixels[i] + ",";
      // google.colab.kernel.invokeFunction(
      // 'notebook.set_image', [result], {});
      const done = document.createElement('a');
      done.href = "pred?img=" + result;
      //done.href = URL.createObjectURL(new Blob([result], {type: 'text/plain'}));
      //done.download = 'img.csv';
      done.style.display = 'none';
      document.body.appendChild(done);
      done.click();
      document.body.removeChild(done);
  }

  function clear_value() {
      canvas.getContext("2d").fillStyle = "rgb(255, 255, 255)";
      canvas.getContext("2d").fillRect(0, 0, 140, 140);
      for (var i = 0; i < 28*28; i++) pixels[i] = 0;
  }
</script>
</body>
</html>
'''
    return html


@app.route('/pred', methods=['GET'])
def page_pred():
    img = flask.request.args.get("img")
    with open('ex5-6/img.csv', 'w') as f:
        f.write(img)
    proc = subprocess.run(['python', 'ex5-6_pred.py', 'nofig'])
    html = '''<!DOCTYPE html>
<html>
<body>
<p><a href="/">next</a></p>
<p>img={}</p>
<h2>written and prediction</h2>
<img src="./ex5-6/ex5-6.png">
<hr>
<p><a href="/">next</a></p>
</body>
</html>
'''.format(img)
    return html


if __name__ == '__main__':
    print(app.url_map)

    if not os.path.isdir('ex5-6'):
        os.mkdir('ex5-6')

    # app.run(port=8000)
    app.run(host='0.0.0.0', port=8056)
