#!/usr/bin/python3
# -*- coding: utf-8 -*-

# The code is derived from https://github.com/enakai00/colab_tfbook
# and modified by izumi@ieee.org, 2021

# このコードはこの↓書籍のサンプルコードをベースに改変したものである
# 『TensorFlow と Keras で 動かしながら学ぶ ディープラーニングの仕組み』
# 中井悦司, マイナビ出版, 2019年
# https://book.mynavi.jp/ec/products/detail/id=109925

import numpy as np
from numpy.random import multivariate_normal, permutation
import pandas as pd
from pandas import DataFrame
import matplotlib.pyplot as plt

import tensorflow as tf
from tensorflow.keras import layers, models

import time
import sys
import os
import specrep
print(*sys.argv)
if 'nogpu' in sys.argv:
    # tf.device('/CPU:0')
    os.environ['CUDA_VISIBLE_DEVICES'] = '-1'
else:
    # tf.device('/GPU:0')
    pass
flag_fig  = False if 'nofig'  in sys.argv else True
flag_spec = False if 'nospec' in sys.argv else True
if flag_spec: specrep.report()
if (not tf.__version__.startswith('2.')):
    print('This code requires version 2.*.')
    exit()


print('# Comparing Accuracy for Training and Test Datasets)')

print('####[CAT-01]\nImport modules and set a random seed.')

np.random.seed(20190825)
tf.random.set_seed(20190825)

print('####[CAT-02]\nGenerate a dateset and split them into training(80%) and test datasets(20%).')

n0, mu0, variance0 = 800, [10, 11], 20
data0 = multivariate_normal(mu0, np.eye(2)*variance0, n0)
df0 = DataFrame(data0, columns=['x1', 'x2'])
df0['t'] = 0

n1, mu1, variance1 = 600, [18, 20], 22
data1 = multivariate_normal(mu1, np.eye(2)*variance1, n1)
df1 = DataFrame(data1, columns=['x1', 'x2'])
df1['t'] = 1

df = pd.concat([df0, df1], ignore_index=True)
df = df.reindex(permutation(df.index)).reset_index(drop=True)

num_data = int(len(df)*0.8)
train_set = df[:num_data]
test_set = df[num_data:]


print('####[CAT-04]\nStore the coordinates $(x_1,x_2)$ and label values $t$ into NumPy arrays.')

train_x = train_set[['x1', 'x2']].values
train_t = train_set['t'].values
test_x = test_set[['x1', 'x2']].values
test_t = test_set['t'].values

print('####[CAT-05]\nDefine a model to predict the probability of `t=1`.')

model = models.Sequential()
model.add(layers.Dense(1, activation='sigmoid', input_shape=(2,),
                       name='logistic_regression'))
model.summary()

print('####[CAT-06]\nCompile the model using the Adam optimizer, and Cross entroy as a loss function.')

model.compile(optimizer='adam',
              loss='binary_crossentropy',
              metrics=['acc'])

print('####[CAT-07]\nApply the optimization for 10 epochs.')

history = model.fit(train_x, train_t,
                    validation_data=(test_x, test_t),
                    batch_size=len(train_x), epochs=10)


print('####[CAT-08]\nApply the optimization for additional 5,000 epochs.')
time_start = time.perf_counter()
history = model.fit(train_x, train_t,
                    validation_data=(test_x, test_t),
                    batch_size=len(train_x), epochs=5000, verbose=0)
time_end = time.perf_counter()
print('time elapsed =', time_end-time_start)

print('####[CAT-09]\nPlot a chart to compare the accuracy for training and test datasets.')

DataFrame({'acc': history.history['acc'],
           'val_acc': history.history['val_acc']}).plot()
plt.savefig('ex2-2a.png')
if flag_fig: plt.show()
