#!/usr/bin/python3
# -*- coding: utf-8 -*-

# The code is derived from https://github.com/enakai00/colab_tfbook
# and modified by izumi@ieee.org, 2021

# このコードはこの↓書籍のサンプルコードをベースに改変したものである
# 『TensorFlow と Keras で 動かしながら学ぶ ディープラーニングの仕組み』
# 中井悦司, マイナビ出版, 2019年
# https://book.mynavi.jp/ec/products/detail/id=109925

import numpy as np
import matplotlib.pyplot as plt

import tensorflow as tf
from tensorflow.keras.datasets import mnist

import time
import sys
import os
import specrep
print(*sys.argv)
if 'nogpu' in sys.argv:
    # tf.device('/CPU:0')
    os.environ['CUDA_VISIBLE_DEVICES'] = '-1'
else:
    # tf.device('/GPU:0')
    pass
flag_fig  = False if 'nofig'  in sys.argv else True
flag_spec = False if 'nospec' in sys.argv else True
if flag_spec: specrep.report()
if (not tf.__version__.startswith('2.')):
    print('This code requires version 2.*.')
    exit()


print('# MNIST Dataset Sample')


print('####[MDS-02]\nDownload the MNIST dataset and store into NumPy arrays.')
(train_images, train_labels), (test_images, test_labels) = mnist.load_data()
print('train_images', type(train_images), train_images.shape)
print('train_labels', type(train_labels), train_labels.shape)
print('test_images ', type(test_images),  test_images.shape)
print('test_labels ', type(test_labels),  test_labels.shape)

print('example: train_images[0]\n', train_images[0])
print('example: train_labels[0]\n', train_labels[0])

print('Reshape the dataset.')

train_images = train_images.reshape(
    (len(train_images), 784)).astype('float32') / 255
test_images = test_images.reshape(
    (len(test_images), 784)).astype('float32') / 255
train_labels = tf.keras.utils.to_categorical(train_labels, 10)
test_labels = tf.keras.utils.to_categorical(test_labels, 10)

print('train_images', type(train_images), train_images.shape)
print('train_labels', type(train_labels), train_labels.shape)
print('test_images ', type(test_images),  test_images.shape)
print('test_labels ', type(test_labels),  test_labels.shape)

print('####[MDS-03]\nShow the first image from the training dataset. It\'s a flat list of 28*28 pixel values.')
print('example: train_images[0]\n', train_images[0])

print('####[MDS-04]\nShow the corresponding label. In this case, the 5th (starting from 0th) element is 1. Hence, the image is "5".')
print('example: train_labels[0]\n', train_labels[0])

print('####[MDS-05]\nShow some examples as images.')
fig = plt.figure(figsize=(8, 4))
for c, (image, label) in enumerate(zip(train_images[:10], train_labels[:10])):
    subplot = fig.add_subplot(2, 5, c+1)
    subplot.set_xticks([])
    subplot.set_yticks([])
    subplot.set_title('%d' % np.argmax(label))
    subplot.imshow(image.reshape((28, 28)),
                   vmin=0, vmax=1, cmap=plt.cm.gray_r)

plt.savefig('ex2-3a.png')
if flag_fig: plt.show()
