#!/usr/bin/python3
# -*- coding: utf-8 -*-

# The code is derived from https://github.com/enakai00/colab_tfbook
# and modified by izumi@ieee.org, 2021

# このコードはこの↓書籍のサンプルコードをベースに改変したものである
# 『TensorFlow と Keras で 動かしながら学ぶ ディープラーニングの仕組み』
# 中井悦司, マイナビ出版, 2019年
# https://book.mynavi.jp/ec/products/detail/id=109925

import numpy as np
from pandas import DataFrame
import matplotlib.pyplot as plt

import tensorflow as tf
from tensorflow.keras import layers, models, initializers
from tensorflow.keras.datasets import mnist

import time
import sys
import os
import specrep
print(*sys.argv)
if 'nogpu' in sys.argv:
    # tf.device('/CPU:0')
    os.environ['CUDA_VISIBLE_DEVICES'] = '-1'
else:
    # tf.device('/GPU:0')
    pass
flag_fig  = False if 'nofig'  in sys.argv else True
flag_spec = False if 'nospec' in sys.argv else True
if flag_spec: specrep.report()
if (not tf.__version__.startswith('2.')):
    print('This code requires version 2.*.')
    exit()


# MNIST Single Layer Network

np.random.seed(20190228)
tf.random.set_seed(20190228)


print('####[MSL-02]\nDownload the MNIST dataset and store into NumPy arrays.')
(train_images, train_labels), (test_images, test_labels) = mnist.load_data()
train_images = train_images.reshape(
    (len(train_images), 784)).astype('float32') / 255
test_images = test_images.reshape(
    (len(test_images), 784)).astype('float32') / 255
train_labels = tf.keras.utils.to_categorical(train_labels, 10)
test_labels = tf.keras.utils.to_categorical(test_labels, 10)


print('####[MSL-03]\nDefine a model with a single hidden layer.')
model = models.Sequential()
model.add(layers.Dense(1024, activation='relu', input_shape=(28*28,),
                       kernel_initializer=initializers.TruncatedNormal(),
                       name='hidden'))
model.add(layers.Dense(10, activation='softmax', name='softmax'))
model.summary()


print('####[MSL-04]\nCompile the model using the Adam optimizer, and Cross entroy as a loss function.')
model.compile(optimizer='adam',
              loss='categorical_crossentropy',
              metrics=['acc'])


print('####[MSL-05]\nTrain the model. It acheves the 98.1% accuracy for the test dataset.')
time_start = time.perf_counter()
history = model.fit(train_images, train_labels,
                    validation_data=(test_images, test_labels),
                    batch_size=128, epochs=10)
time_end = time.perf_counter()
print('time elapsed =', time_end-time_start)

DataFrame({'acc': history.history['acc'],
           'val_acc': history.history['val_acc']}).plot()
plt.savefig('ex3-2a.png')
if flag_fig: plt.show()
DataFrame({'loss': history.history['loss']}).plot()
plt.savefig('ex3-2b.png')
if flag_fig: plt.show()


print('####[MSL-06]\nShow examples of the prediction result. Three for correct preditions and three for incorrect predictions.')
p_val = model.predict(np.array(test_images))
df = DataFrame({'pred': list(map(np.argmax, p_val)),
                'label': list(map(np.argmax, test_labels))})
correct = df[df['pred'] == df['label']]
incorrect = df[df['pred'] != df['label']]

fig = plt.figure(figsize=(8, 15))
for i in range(10):
    indices = list(correct[correct['pred'] == i].index[:3]) \
        + list(incorrect[incorrect['pred'] == i].index[:3])
    for c, image in enumerate(test_images[indices]):
        subplot = fig.add_subplot(10, 6, i*6+c+1)
        subplot.set_xticks([])
        subplot.set_yticks([])
        subplot.set_title('%d / %d' % (i, df['label'][indices[c]]))
        subplot.imshow(image.reshape((28, 28)),
                       vmin=0, vmax=1, cmap=plt.cm.gray_r)

plt.savefig('ex3-2.png')
if flag_fig: plt.show()
