#!/usr/bin/python3
# -*- coding: utf-8 -*-

# The code is derived from https://github.com/enakai00/colab_tfbook
# and modified by izumi@ieee.org, 2021

# このコードはこの↓書籍のサンプルコードをベースに改変したものである
# 『TensorFlow と Keras で 動かしながら学ぶ ディープラーニングの仕組み』
# 中井悦司, マイナビ出版, 2019年
# https://book.mynavi.jp/ec/products/detail/id=109925

import numpy as np
import shutil
import socket

import tensorflow as tf
from tensorflow.keras import layers, models, initializers, callbacks
from tensorflow.keras.datasets import mnist

import time
import sys
import os
import specrep
print(*sys.argv)
if 'nogpu' in sys.argv:
    # tf.device('/CPU:0')
    os.environ['CUDA_VISIBLE_DEVICES'] = '-1'
else:
    # tf.device('/GPU:0')
    pass
flag_fig  = False if 'nofig'  in sys.argv else True
flag_spec = False if 'nospec' in sys.argv else True
if flag_spec: specrep.report()
if (not tf.__version__.startswith('2.')):
    print('This code requires version 2.*.')
    exit()


np.random.seed(20190228)
tf.random.set_seed(20190228)


print('####[MST-02]\nDownload the MNIST dataset and store into NumPy arrays.')
(train_images, train_labels), (test_images, test_labels) = mnist.load_data()
train_images = train_images.reshape(
    (len(train_images), 784)).astype('float32') / 255
test_images = test_images.reshape(
    (len(test_images), 784)).astype('float32') / 255
train_labels = tf.keras.utils.to_categorical(train_labels, 10)
test_labels = tf.keras.utils.to_categorical(test_labels, 10)


print('####[MST-03]\nDefine a model with a single hidden layer.')
model = models.Sequential()
model.add(layers.Dense(1024, activation='relu', input_shape=(28*28,),
                       kernel_initializer=initializers.TruncatedNormal(),
                       name='hidden'))
model.add(layers.Dense(10, activation='softmax', name='softmax'))
model.summary()


print('####[MST-04]\nCompile the model using the Adam optimizer, and Cross entroy as a loss function.')
model.compile(optimizer='adam',
              loss='categorical_crossentropy',
              metrics=['acc'])


print('####[MST-05]\nTrain the model with the callbacks option to store training logs.')
log_dir = 'ex3-3_log'
shutil.rmtree(log_dir, ignore_errors=True)
tensorboard_callback = callbacks.TensorBoard(log_dir=log_dir,
                                             histogram_freq=1)
time_start = time.perf_counter()
history = model.fit(train_images, train_labels,
                    validation_data=(test_images, test_labels),
                    batch_size=128, epochs=10,
                    callbacks=[tensorboard_callback])
time_end = time.perf_counter()
print('time elapsed =', time_end-time_start)


print('####\nStart TensorBoard and connect to the URL.')

print('(1) To provide log viewer, run the following commands on your computer.')
print('$ tensorboard --logdir {} --host 0.0.0.0 --port 6006'.format(log_dir))
# host 0.0.0.0 accepts access from any other host

s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
s.connect(("133.19.170.14", 80))
print('(2) To check the log, connect to http://{}:6006 with your web browser.'.format(s.getsockname()[0]))
