#!/usr/bin/python3
# -*- coding: utf-8 -*-

# The code is derived from https://github.com/enakai00/colab_tfbook
# and modified by izumi@ieee.org, 2021

# このコードはこの↓書籍のサンプルコードをベースに改変したものである
# 『TensorFlow と Keras で 動かしながら学ぶ ディープラーニングの仕組み』
# 中井悦司, マイナビ出版, 2019年
# https://book.mynavi.jp/ec/products/detail/id=109925

import numpy as np
from pandas import DataFrame
import matplotlib.pyplot as plt

import tensorflow as tf
from tensorflow.keras import layers, models, initializers
from tensorflow.keras.datasets import mnist

import time
import sys
import os
import specrep
print(*sys.argv)
if 'nogpu' in sys.argv:
    # tf.device('/CPU:0')
    os.environ['CUDA_VISIBLE_DEVICES'] = '-1'
else:
    # tf.device('/GPU:0')
    pass
flag_fig  = False if 'nofig'  in sys.argv else True
flag_spec = False if 'nospec' in sys.argv else True
if flag_spec: specrep.report()
if (not tf.__version__.startswith('2.')):
    print('This code requires version 2.*.')
    exit()

print('# MNIST Double Layer CNN Classification')

np.random.seed(20190213)
tf.random.set_seed(20190213)


print('####[CNN-02]\nDownload the MNIST dataset and store into NumPy arrays.')
(train_images, train_labels), (test_images, test_labels) = mnist.load_data()
train_images = train_images.reshape(
    (len(train_images), 784)).astype('float32') / 255
test_images = test_images.reshape(
    (len(test_images), 784)).astype('float32') / 255
train_labels = tf.keras.utils.to_categorical(train_labels, 10)
test_labels = tf.keras.utils.to_categorical(test_labels, 10)


print('####[CNN-03]\nDefine a CNN model with a single convolutional filter layer.')


model = models.Sequential()
model.add(layers.Reshape((28, 28, 1), input_shape=(28*28,), name='reshape'))
model.add(layers.Conv2D(32, (5, 5), padding='same',
                        kernel_initializer=initializers.TruncatedNormal(),
                        use_bias=True, activation='relu',
                        name='conv_filter1'))
model.add(layers.MaxPooling2D((2, 2), name='max_pooling1'))
model.add(layers.Conv2D(64, (5, 5), padding='same',
                        kernel_initializer=initializers.TruncatedNormal(),
                        use_bias=True, activation='relu',
                        name='conv_filter2'))
model.add(layers.MaxPooling2D((2, 2), name='max_pooling2'))
model.add(layers.Flatten(name='flatten'))
model.add(layers.Dense(1024, activation='relu',
                       kernel_initializer=initializers.TruncatedNormal(),
                       name='hidden'))
model.add(layers.Dropout(rate=0.5, name='dropout'))
model.add(layers.Dense(10, activation='softmax', name='softmax'))

model.summary()


print('####[CNN-04]\nCompile the model using the Adam optimizer, and Cross entroy as a loss function.')
model.compile(optimizer='adam',
              loss='categorical_crossentropy',
              metrics=['acc'])


print('####[CNN-05]\nTrain the model. It achieves the 99.4% accuracy.')
time_start = time.perf_counter()
history = model.fit(train_images, train_labels,
                    validation_data=(test_images, test_labels),
                    batch_size=128, epochs=10)
time_end = time.perf_counter()
print('time elapsed =', time_end-time_start)

print('####[CNN-06]\nPlot chars for accuracy and loss values.')
DataFrame({'acc': history.history['acc'],
           'val_acc': history.history['val_acc']}).plot()
plt.savefig('ex5-1a.png')
if flag_fig: plt.show()
DataFrame({'loss': history.history['loss'],
           'val_loss': history.history['val_loss']}).plot()
plt.savefig('ex5-1b.png')
if flag_fig: plt.show()


print('####[MDF-08]\nExport the trained model as a file `MNIST_double.hd5`')
model.save('MNIST_double.hd5', save_format='h5')
