#!/usr/bin/python3
# -*- coding: utf-8 -*-

# The code is derived from https://github.com/enakai00/colab_tfbook
# and modified by izumi@ieee.org, 2021

# このコードはこの↓書籍のサンプルコードをベースに改変したものである
# 『TensorFlow と Keras で 動かしながら学ぶ ディープラーニングの仕組み』
# 中井悦司, マイナビ出版, 2019年
# https://book.mynavi.jp/ec/products/detail/id=109925

import numpy as np
import matplotlib.pyplot as plt

import tensorflow as tf
from tensorflow.keras import layers, models

import time
import sys
import os
import specrep
print(*sys.argv)
if 'nogpu' in sys.argv:
    # tf.device('/CPU:0')
    os.environ['CUDA_VISIBLE_DEVICES'] = '-1'
else:
    # tf.device('/GPU:0')
    pass
flag_fig  = False if 'nofig'  in sys.argv else True
flag_spec = False if 'nospec' in sys.argv else True
if flag_spec: specrep.report()
if (not tf.__version__.startswith('2.')):
    print('This code requires version 2.*.')
    exit()


print('# Handwritten Digit Recognizer')


print('####[HDR-04]\nRestore the model from the exported model file.')
model = models.load_model('MNIST_double.hd5')
model.summary()


print('####[HDR-05]\nDefine a model to extract outputs from intermediate layers.')
layer_outputs = [model.get_layer('max_pooling1').output,
                 model.get_layer('max_pooling2').output]
model2 = models.Model(inputs=model.input, outputs=layer_outputs)


print('####\nHow to recognize your handwriting.')

print('1. open ex5-2.html by your web browser.')
print('2. write a number in the area.')
print('3. download the image as img.csv.')
print('4. move downloaded img.csv to ./ex5-2/.')
print('5. execute ex5-2.py.')

if not os.path.isfile('ex5-2/img.csv'):
    print('img.csv is not here.')
    exit()

with open('ex5-2/img.csv') as f:
    image = np.array(f.read().split(',')[:(28*28)], dtype=np.float32)

fig = plt.figure(figsize=(3, 3))
subplot = fig.add_subplot(1, 1, 1)
subplot.set_xticks([])
subplot.set_yticks([])
subplot.imshow(image.reshape((28, 28)),
               vmin=0, vmax=1, cmap=plt.cm.gray_r)
plt.savefig('ex5-2/ex5-2a.png')
if flag_fig: plt.show()


print('####[HDR-08]\nMake a prediction for the handwritten digit on the drawing canvas.')
time_start = time.perf_counter()
p_val = model.predict(image.reshape(1, 28*28))
time_end = time.perf_counter()
print('time elapsed =', time_end-time_start)

fig = plt.figure(figsize=(6, 3))
pred = p_val[0]
print('pred:', type(pred), pred.shape)
subplot = fig.add_subplot(1, 1, 1)
subplot.set_xticks(range(10))
subplot.set_xlim(-0.5, 9.5)
subplot.set_ylim(0, 1)
subplot.bar(range(10), pred, align='center')
plt.savefig('ex5-2/ex5-2b.png')
if flag_fig: plt.show()


print('####[HDR-09]\nGet output images from the convolutional filters.')
pool1_output, pool2_output = model2.predict(image.reshape(1, 28*28))


print('####[HDR-10]\nShow output images from the first pooling layer.')
layer = pool1_output
fig = plt.figure(figsize=(20, 10))
for i in range(layer.shape[-1]):
    subplot = fig.add_subplot(8, 16, i+1)
    subplot.set_xticks([])
    subplot.set_yticks([])
    subplot.imshow(layer[0, :, :, i], cmap=plt.cm.gray_r)
plt.savefig('ex5-2/ex5-2c.png')
if flag_fig: plt.show()


print('####[HDR-11]\nShow output images from the second pooling layer.')
layer = pool2_output
fig = plt.figure(figsize=(20, 10))
for i in range(layer.shape[-1]):
    subplot = fig.add_subplot(8, 16, i+1)
    subplot.set_xticks([])
    subplot.set_yticks([])
    subplot.imshow(layer[0, :, :, i], cmap=plt.cm.gray_r)
plt.savefig('ex5-2/ex5-2d.png')
if flag_fig: plt.show()
