#!/usr/bin/python3
# -*- coding: utf-8 -*-

# The code is derived from https://github.com/enakai00/colab_tfbook
# and modified by izumi@ieee.org, 2021

# このコードはこの↓書籍のサンプルコードをベースに改変したものである
# 『TensorFlow と Keras で 動かしながら学ぶ ディープラーニングの仕組み』
# 中井悦司, マイナビ出版, 2019年
# https://book.mynavi.jp/ec/products/detail/id=109925

import numpy as np
import matplotlib.pyplot as plt
import cv2

import tensorflow as tf
from tensorflow.keras import backend as K
from tensorflow.keras import layers, models, preprocessing
from tensorflow.keras.datasets import mnist
from tensorflow.python.framework.ops import disable_eager_execution

import time
import sys
import os
import specrep
print(*sys.argv)
if 'nogpu' in sys.argv:
    # tf.device('/CPU:0')
    os.environ['CUDA_VISIBLE_DEVICES'] = '-1'
else:
    # tf.device('/GPU:0')
    pass
flag_fig  = False if 'nofig'  in sys.argv else True
flag_spec = False if 'nospec' in sys.argv else True
if flag_spec: specrep.report()
if (not tf.__version__.startswith('2.')):
    print('This code requires version 2.*.')
    exit()


print('# CNN Interpretation - Sensitive Area Detection')

disable_eager_execution()


print('####[CIS-02]\nDefine a function to detect the sensitive areas in the second filters against the input image, and generate a heatmap.')
def get_heatmap(model, last_conv_layer, target_image):
    # print('get_heatmap()')
    # print('model:', type(model))  # keras.engine.sequential.Sequential
    # print('last_conv_layer:', type(last_conv_layer))  # keras.layers.convolutional.Conv2D
    # print('target_image:', type(target_image), target_image.shape)  # ndarray (784)
    target_image = target_image.reshape(1, 28*28)  # 計算の実行

    # pout = model.predict(target_image)
    # mout = model.output
    # print('pout:', type(pout), pout.shape)  # ndarray (1,10)
    # print('mout:', type(mout), mout.shape)  # Tensor (None, 10)
    # print('mout[0]:', type(mout[0]), mout[0].shape, mout[0])  # Tensor (10)
    # model.predict は target_image に対する計算の実行、出力は確定値
    # model.output は計算式の登録（まだ計算はしない）
    pred_output = model.output[:, np.argmax(model.predict(target_image))]  # argmax(model.predict()) は計算の実行で、それ以外は計算式の登録
    # この target_image の最大値となる確率だけを取り出す計算式
    # print('pred_output:', type(pred_output), pred_output.shape)  # Tensor (None)  各ターゲットに対するスカラの出力

    # x = K.gradients(pred_output, last_conv_layer.output)
    # print('K.gradients:', type(x), len(x))  #
    grads = K.gradients(pred_output, last_conv_layer.output)[0]  # 計算式の登録
    # print('grads:', type(grads), grads.shape)  # Tensor (None, 14, 14, 64)

    pooled_grads = K.mean(grads, axis=(0, 1, 2))  # 計算式の登録
    # print('pooled_grads:', type(pooled_grads), pooled_grads.shape)  # Tensor (64)
    get_vals = K.function([model.input],
                          [pooled_grads, last_conv_layer.output[0]])  # 計算式の登録
    pooled_grads_val, conv_layer_output_val = get_vals([target_image])  # 計算の実行
    # print('pooled_grads_val:', type(pooled_grads_val), pooled_grads_val.shape)  # ndarray (64)
    # print('conv_layer_output_val:', type(conv_layer_output_val), conv_layer_output_val.shape)  # ndarray (14,14,64)

    heatmap = np.zeros([14, 14])
    for i in range(last_conv_layer.output.shape[-1]):
        if pooled_grads_val[i] > 0:
            heatmap += conv_layer_output_val[:, :, i] * pooled_grads_val[i]
    heatmap /= np.max(heatmap)

    heatmap2 = np.uint8(255 * cv2.resize(heatmap, (28, 28)))
    heatmap2 = cv2.applyColorMap(heatmap2, cv2.COLORMAP_JET)
    target_image2 = np.uint8(255 * target_image.reshape(28, 28))
    target_image2 = cv2.applyColorMap(target_image2, cv2.COLORMAP_BONE)
    superimposed_img = heatmap2 + target_image2 * 0.6
    cv2.imwrite('tmp_ex5-4.jpg', superimposed_img)

    return heatmap, preprocessing.image.load_img('tmp_ex5-4.jpg')


print('####[CIS-03]\nDownload the MNIST dataset and store into NumPy arrays.')
(train_images, train_labels), (test_images, test_labels) = mnist.load_data()
print('train_images:', type(train_images), train_images.shape)  # ndarray (60000,28,28)
print('train_labels:', type(train_labels), train_labels.shape)  # ndarray (60000,10)

print('reshaping data...')
train_images = train_images.reshape(
    (len(train_images), 784)).astype('float32') / 255
test_images = test_images.reshape(
    (len(test_images), 784)).astype('float32') / 255
train_labels = tf.keras.utils.to_categorical(train_labels, 10)
test_labels = tf.keras.utils.to_categorical(test_labels, 10)
print('train_images:', type(train_images), train_images.shape)  # ndarray (60000,784)
print('train_labels:', type(train_labels), train_labels.shape)  # ndarray (60000,10)


print('####[CIS-06]\nRestore the model from the exported model file.')
model = models.load_model('MNIST_double.hd5')
model.summary()


print('####[CIS-07]\nShow heatmaps for sample images.')
fig = plt.figure(figsize=(8, 8))
last_conv_layer = model.get_layer('conv_filter2')
i = 1
for img in test_images[[3, 5, 1, 32, 4, 15, 21, 254, 84, 12]]:
    heatmap, superimposed = get_heatmap(model, last_conv_layer, img)

    subplot = fig.add_subplot(5, 6, i)
    subplot.set_xticks([])
    subplot.set_yticks([])
    subplot.imshow(img.reshape(28, 28), vmin=0, vmax=1, cmap=plt.cm.gray_r)

    subplot = fig.add_subplot(5, 6, i+1)
    subplot.set_xticks([])
    subplot.set_yticks([])
    subplot.imshow(heatmap, vmin=0, vmax=1, cmap=plt.cm.gray_r)

    subplot = fig.add_subplot(5, 6, i+2)
    subplot.set_xticks([])
    subplot.set_yticks([])
    subplot.imshow(superimposed)

    i += 3
plt.savefig('ex5-4.png')
if flag_fig: plt.show()
