#!/usr/bin/python3
# -*- coding: utf-8 -*-

# The code is derived from https://github.com/enakai00/colab_tfbook
# and modified by izumi@ieee.org, 2021

# このコードはこの↓書籍のサンプルコードをベースに改変したものである
# 『TensorFlow と Keras で 動かしながら学ぶ ディープラーニングの仕組み』
# 中井悦司, マイナビ出版, 2019年
# https://book.mynavi.jp/ec/products/detail/id=109925

import numpy as np
import matplotlib.pyplot as plt

import tensorflow as tf
from tensorflow.keras import layers, models
from tensorflow.keras.datasets import mnist

import time
import sys
import os
import specrep
print(*sys.argv)
if 'nogpu' in sys.argv:
    # tf.device('/CPU:0')
    os.environ['CUDA_VISIBLE_DEVICES'] = '-1'
else:
    # tf.device('/GPU:0')
    pass
flag_fig  = False if 'nofig'  in sys.argv else True
flag_spec = False if 'nospec' in sys.argv else True
if flag_spec: specrep.report()
if (not tf.__version__.startswith('2.')):
    print('This code requires version 2.*.')
    exit()


print('# MNIST Autoencoder with handwriting')

print('####[HDR-04]\nRestore the model from the exported model file.')
model = models.load_model('MNIST_autoenc.hd5')
model.summary()


print('####\nHow to apply your handwriting to autoencoder.')

print('1. open ex5-6.html by your web browser.')
print('2. write a number in the area.')
print('3. download the image as img.csv.')
print('4. move downloaded img.csv to ./ex5-6/.')
print('5. execute ex5-6_pred.py.')

if not os.path.isfile('ex5-6/img.csv'):
    print('img.csv is not here.')
    exit()

with open('ex5-6/img.csv') as f:
    image = np.array(f.read().split(',')[:(28*28)], dtype=np.float32)

fig = plt.figure(figsize=(3, 3))
subplot = fig.add_subplot(1, 1, 1)
subplot.set_xticks([])
subplot.set_yticks([])
subplot.imshow(image.reshape((28, 28)),
               vmin=0, vmax=1, cmap=plt.cm.gray_r)
plt.savefig('ex5-6/img.png')
if flag_fig: plt.show()


print('####[MAE-10]\nShow a resconstructed image from the handwritten example.')
time_start = time.perf_counter()
results = model.predict(np.array([image]))
time_end = time.perf_counter()
print('time elapsed =', time_end-time_start)

fig = plt.figure(figsize=(6, 3))
subplot = fig.add_subplot(1, 2, 1)
subplot.set_xticks([])
subplot.set_yticks([])
subplot.imshow(np.array(image).reshape((28, 28)),
               vmin=0, vmax=1, cmap=plt.cm.gray_r)
subplot = fig.add_subplot(1, 2, 2)
subplot.set_xticks([])
subplot.set_yticks([])
subplot.imshow(results[0].reshape((28, 28)),
               vmin=0, vmax=1, cmap=plt.cm.gray_r)

plt.savefig('ex5-6/ex5-6.png')
if flag_fig: plt.show()
