#!/usr/bin/python3
# -*- coding: utf-8 -*-

# The code is derived from https://github.com/enakai00/colab_tfbook
# and modified by izumi@ieee.org, 2021

# このコードはこの↓書籍のサンプルコードをベースに改変したものである
# 『TensorFlow と Keras で 動かしながら学ぶ ディープラーニングの仕組み』
# 中井悦司, マイナビ出版, 2019年
# https://book.mynavi.jp/ec/products/detail/id=109925


import numpy as np
import matplotlib.pyplot as plt
import matplotlib.animation as animation
from PIL import Image

from tensorflow.keras import models
import tensorflow as tf

import time
import sys
import os
import specrep
print(*sys.argv)
if 'nogpu' in sys.argv:
    # tf.device('/CPU:0')
    os.environ['CUDA_VISIBLE_DEVICES'] = '-1'
else:
    # tf.device('/GPU:0')
    pass
flag_fig  = False if 'nofig'  in sys.argv else True
flag_spec = False if 'nospec' in sys.argv else True
if flag_spec: specrep.report()
if (not tf.__version__.startswith('2.')):
    print('This code requires version 2.*.')
    exit()


print('# MNIST DCGAN Example - Result')


print('####[MDR-04]\nRestore the model from the exported model file.')
model = models.load_model('gan_generator.hd5')
model.summary()


print('####[MDR-05]\nDefine a function to show generated images.')
def show_images(images):
    fig = plt.figure(figsize=(len(images)*1.4, 1.4))
    for c, image in enumerate(images):
        subplot = fig.add_subplot(1, len(images), c+1)
        subplot.set_xticks([])
        subplot.set_yticks([])
        subplot.imshow(image.reshape((28, 28)), vmin=0, vmax=1,
                       cmap=plt.cm.gray_r, interpolation='bilinear')


print('####[MDR-06]\nShow some random images.')
latent_dim = 64
random_inputs = np.random.rand(8, latent_dim) * 2.0 - 1.0
show_images(model.predict(random_inputs))
plt.savefig('ex5-8a.png')
if flag_fig: plt.show()
plt.clf()
plt.close()


print('####[MDR-07]\nPick up two examples and generate interporated images.')
start, end = random_inputs[0], random_inputs[7]

interpolation = [np.linspace(s, e, 10) for s, e in zip(start, end)]
example = np.array([[x[c] for x in interpolation] for c in range(10)])
show_images(model.predict(example))
plt.savefig('ex5-8b.png')
if flag_fig: plt.show()
plt.clf()
plt.close()


print('####[MDR-08]\nCreate an animation of dancing digits.')
x = np.random.rand(3, latent_dim) * 2.0 - 1.0
inputs = []
theta1, theta2 = 0, 0
for _ in range(64):
    theta1 += 2*np.pi / 64 * 2
    theta2 += 2*np.pi / 64
    inputs.append(np.cos(theta1)*x[0] + np.sin(theta1)*(np.cos(theta2)*x[1]+np.sin(theta2)*x[2]))
res = model.predict(np.array(inputs))

fig = plt.figure(figsize=(8, 8))
for c, image in enumerate(res):
    subplot = fig.add_subplot(8, 8, c+1)
    subplot.set_xticks([])
    subplot.set_yticks([])
    subplot.imshow(image.reshape((28, 28)), vmin=0, vmax=1,
                   cmap=plt.cm.gray_r, interpolation='bilinear')
plt.savefig('ex5-8c.png')
if flag_fig: plt.show()
plt.clf()
plt.close()

if not os.path.isdir('ex5-8'):
    os.mkdir('ex5-8')

for c, image in enumerate(res):
    im = Image.fromarray(np.uint8(255.0*(1.0-image.reshape(28, 28))))
    fn = 'ex5-8/{:02}.png'.format(c)
    im.convert('RGB')
    im.save(fn)
plt.clf()
plt.close()


images = []
fig = plt.figure(figsize=(2, 2))
for i in range(len(res)):
    plt.xticks([])
    plt.yticks([])
    image = plt.imshow(res[i].reshape((28, 28)), vmin=0, vmax=1,
                       cmap=plt.cm.gray_r, interpolation='bilinear')
    images.append([image])

ani = animation.ArtistAnimation(fig, images, interval=10, repeat=True)
ani.save('DCGAN_morphing.gif', writer='pillow', fps=100)

print('See "DCGAN_morphing.gif"')
