#!/usr/bin/python3
# -*- coding: utf-8 -*-
# 　

import sys
import time
import psutil
import subprocess
import numpy as np
import tensorflow as tf
from tensorflow.python.client import device_lib

def shcmd(cmd):
    print('\n$', *cmd)
    try:
        proc = subprocess.run(cmd, stdout=subprocess.PIPE)
        print(proc.stdout.decode('utf-8'))
    except FileNotFoundError:
        print('not found')
    return

def report():
    print('', '-'*64, '', sep='\n')
    t = time.localtime()
    print('executed at {:4}/{:02}/{:02} {:02}:{:02}:{:02}'.format(t.tm_year, t.tm_mon, t.tm_mday, t.tm_hour, t.tm_min, t.tm_sec))

    print('', '-'*64, '-  versions', '', sep='\n')
    print('python version:', sys.version)
    print('numpy version:', np.__version__)
    print('tensorflow version: ', tf.__version__)
    print(device_lib.list_local_devices())

    print('', '-'*64, '-  specifications of the computer', '', sep='\n')
    print('CPU cores:', psutil.cpu_count(logical=False))
    print('memory:', psutil.virtual_memory().total)

    shcmd(['hostname'])
    shcmd(['whoami'])
    shcmd(['cat', '/proc/cpuinfo'])
    shcmd(['cat', '/proc/meminfo'])
    shcmd(['lsb_release', '-a'])
    shcmd(['uname', '-a'])
    shcmd(['sh', '-c', 'lspci | grep -i vga'])
    shcmd(['nvidia-smi'])
    shcmd(['nvcc', '-V'])
    shcmd(['grep', '-A', '2', 'CUDNN_MAJOR', '/usr/local/cuda/include/cudnn_version.h'])


if __name__ == '__main__':
    report()
